#include <TMPM330.H>                    // TMPM330 definitions

/* Fijarse en Project -> Options -> C/C++ Compiler -> Preprocessor
los paths para agregar el CMSIS y que el IAR lo incluya 
est en $TOOLKIT_DIR$/CMSIS porque yo (SRC) lo puse ahi */

#include <math.h>
#define m_PI 3.14159265358979323846264338327
#include <stdio.h>


/* LCD control signals */
#define LCD_RSTb 3
#define	LCD_RDb	4
#define	LCD_A0b	5
#define	LCD_WRb	6
#define	LCD_CSb	7

#define LCD_RST TSB_PI_DATA_PI3
#define	LCD_RD	TSB_PI_DATA_PI4
#define	LCD_A0	TSB_PI_DATA_PI5
#define	LCD_WR	TSB_PI_DATA_PI6
#define	LCD_CS	TSB_PI_DATA_PI7


volatile unsigned long SysTickCnt;      // SysTick Counter

// SysTick Setup
void SysTick_Setup (void)
{
  SysTick->LOAD = 40000 - 1;            // 1ms: 40000 ticks @ 40MHz
  SysTick->CTRL = 0x0007;               // Enable Counter and Tick Interrupt
}

// SysTick Interrupt Handler (tick each 1ms)
void SysTick_Handler (void)
{
  SysTickCnt++;
}


// Delay Function
void Delay (unsigned long tick)
{
unsigned long systickcnt;

  systickcnt = SysTickCnt;
  while ((SysTickCnt - systickcnt) < tick);
}
#define MsDelay Delay

//                R G B
#define RED	0xff0000
#define GREEN	0x00ff00
#define BLUE	0x0000ff
#define BLACK	0x000000
#define WHITE	0xffffff
#define GRAY	0x808080
#define CYAN	0xffff00

#define MAKECOLOR(a,b,c)	((a<<16)+(b<<8)+c)


/* Low level functions */
void LCD_Write(uint8_t byte)
{
	TSB_PJ->DATA = byte;							// data out
	LCD_CS=0;
	LCD_WR=0;
	LCD_WR=1;
	LCD_CS=1;
}

__INLINE void LCD_WriteFast(uint8_t byte)
{
	TSB_PJ->DATA = byte;							// data out
	LCD_WR=0;
	LCD_WR=1;
}

void LCD_WritePixel(uint32_t rgb)
{
	LCD_CS=0;
	LCD_WriteFast(((uint32_t)rgb>>16)&0xFF);
	LCD_WriteFast(((uint32_t)rgb>>8)&0xFF);
	LCD_WriteFast(((uint32_t)rgb>>0)&0xFF);
	LCD_CS=1;
}

void LCD_WriteArea(uint32_t rgb,uint32_t area)
{
	LCD_CS=0;
	while(area--){
		LCD_WriteFast(((uint32_t)rgb>>16)&0xFF);	// LCD_WritePixel(rgb)
		LCD_WriteFast(((uint32_t)rgb>>8)&0xFF);
		LCD_WriteFast(((uint32_t)rgb>>0)&0xFF);
	}
	LCD_CS=1;
}

void LCD_DumpArea(uint8_t *img,uint32_t area)
{
	LCD_CS=0;
	while(area--){
		LCD_WriteFast(*img++);
		LCD_WriteFast(*img++);
		LCD_WriteFast(*img++);
	}
	LCD_CS=1;
}

void LCD_WriteCmd(uint8_t cmd)
{
	LCD_A0=0;			// Lower A0 (Cmd)
	LCD_Write(cmd);
	LCD_A0=1;			// Rise A0 (Data)
}

void LCD_WriteStrCmd(const uint8_t *cmd,unsigned int len)
{
	LCD_WriteCmd(*cmd++);
	while(len--) {
		LCD_Write(*cmd++);
	}
}

void LCD_setwindow (uint16_t x0,uint16_t x1,uint16_t y0,uint16_t y1 );


void LCD_init ()
{
const static uint8_t init_string1[]={
0xE0,0x01		// START PLL
};
const static uint8_t init_string2[]={
0xE0,0x03		// LOCK PLL
};

const static uint8_t init_string3[]={
0xB0,			//SET LCD MODE  SET TFT 18Bits MODE
0x0C,0x80,		//SET TFT MODE & hsync+Vsync+DEN MODE
0x02,0x7F,		//SET horizontal size=640-1
0x01,0xDF,		//SET vertical size=480-1
0x00 			//SET even/odd line RGB seq.=RGB
};

const static uint8_t init_string4[]={
0xF0,0x00 		//SET pixel data I/F format=8bit
};

const static uint8_t init_string5[]={
0x3A,0x60		// SET R G B format = 6 6 6
};

const static uint8_t init_string6[]={
0xE6,0x02,0xFF,0xFF	//SET PCLK freq=4.94MHz  ; pixel clock frequency
};

const static uint8_t init_string7[]={
0xB4,			//SET HBP,
0x02,0xF8,		//SET HSYNC Total=760
0x00,0x44,		//SET HBP 68
0x0F,			//SET VBP 16=15+1
0x00,0x00,		//SET Hsync pulse start position
0x00			//SET Hsync pulse subpixel start position
};

const static uint8_t init_string8[]={
0xB6,			//SET VBP,
0x01,0xF8,		//SET Vsync total
0x00,0x13,		//SET VBP=19
0x07,			//SET Vsync pulse 8=7+1
0x00,0x00		//SET Vsync pulse start position
};


	MsDelay ( 1000 );
	LCD_RST=0;					// Lower RST
	MsDelay ( 10 );
	LCD_RST=1;					// Raise RST
	MsDelay ( 100 );

	LCD_WriteCmd ( 0x01 );		// Software reset
	LCD_WriteCmd ( 0x01 );		// Software reset
	LCD_WriteCmd ( 0x01 );		// Software reset
	MsDelay ( 100 );
	LCD_WriteStrCmd ( init_string1,sizeof(init_string1) );
	LCD_WriteStrCmd ( init_string2,sizeof(init_string2) );
	MsDelay ( 10 );
	LCD_WriteStrCmd ( init_string3,sizeof(init_string3) );
	LCD_WriteStrCmd ( init_string4,sizeof(init_string4) );
	LCD_WriteStrCmd ( init_string5,sizeof(init_string5) );
	LCD_WriteStrCmd ( init_string6,sizeof(init_string6) );
	LCD_WriteStrCmd ( init_string7,sizeof(init_string7) );
	LCD_WriteStrCmd ( init_string8,sizeof(init_string8) );
	LCD_setwindow ( 0, 639, 0, 479);
	LCD_WriteCmd ( 0x29 );		// Display ON
}

/* High level functions */
void LCD_setwindow ( uint16_t x0, uint16_t x1, uint16_t y0, uint16_t y1 )
{
	LCD_WriteCmd(0x2A);									// SET column address
	LCD_Write(((unsigned)x0>>8)&0xFF);					// HI byte
	LCD_Write(x0&0xFF); 								// LO byte
	LCD_Write(((unsigned)x1>>8)&0xFF);					// HI byte
	LCD_Write(x1&0xFF); 								// LO byte
	LCD_WriteCmd(0x2B);									// SET page address
	LCD_Write(((unsigned)y0>>8)&0xFF);					// HI byte
	LCD_Write(y0&0xFF); 								// LO byte
	LCD_Write(((unsigned)y1>>8)&0xFF);					// HI byte
	LCD_Write(y1&0xFF); 								// LO byte
}


void LCD_fill(uint16_t x, uint16_t y, uint32_t fcolor)
{
	LCD_WriteCmd(0x2C);
  	LCD_WriteArea(fcolor,(uint32_t)x * y);
}

#define LCD_clear() 	LCD_setwindow(0,639,0,479);LCD_fill(640,480,BLACK)


void LCD_icon(uint16_t x, uint16_t y, uint16_t wx, uint16_t wy, uint8_t *img)
{
	LCD_setwindow ( x, wx+x-1, y, wy+y-1);
	LCD_WriteCmd(0x2C);
  	LCD_DumpArea(img,(uint32_t)wx * wy);
}

void QUADS(void)
{
	LCD_setwindow ( 0, 319, 0, 239);
	LCD_WriteCmd(0x2C);
  	LCD_WriteArea(RED,320UL*240);	//
	LCD_setwindow ( 320, 639, 0, 239);
	LCD_WriteCmd(0x2C);
  	LCD_WriteArea(BLUE,320UL*240);	//
	LCD_setwindow ( 0, 319, 240, 479);
	LCD_WriteCmd(0x2C);
  	LCD_WriteArea(CYAN,320UL*240);	//
	LCD_setwindow ( 320, 639, 240, 479);
	LCD_WriteCmd(0x2C);
  	LCD_WriteArea(GREEN,320UL*240);	//
	LCD_setwindow ( 0, 639, 0, 479);
}


void LCD_plot(uint16_t x,uint16_t y,uint32_t pcolor)
{
	LCD_setwindow (x,x,y,y);
	LCD_WriteCmd(0x2C);
	LCD_WritePixel(pcolor);
}

// Bresenham Line Algorithm

void LCD_line(int x1,int y1,int x2,int y2, uint32_t lcolor)
{
int  x,y,dx,dy,incx,incy,balance;

        if (x2 >= x1){
                dx = x2 - x1;
                incx = 1;
        }
        else {
                dx = x1 - x2;
                incx = -1;
        }

        if (y2 >= y1) {
                dy = y2 - y1;
                incy = 1;
        }
        else {
                dy = y1 - y2;
                incy = -1;
        }

        x = x1;
        y = y1;

        if (dx >= dy) {
                dy <<= 1;
                balance = dy - dx;
                dx <<= 1;

                while (x != x2) {
                        LCD_plot(x, y,lcolor);
                        if (balance >= 0) {
                                y += incy;
                                balance -= dx;
                        }
                        balance += dy;
                        x += incx;
                }
                LCD_plot(x, y,lcolor);
        }
        else {
                dx <<= 1;
                balance = dx - dy;
                dy <<= 1;

                while (y != y2) {
                        LCD_plot(x, y,lcolor);
                        if (balance >= 0) {
                                x += incx;
                                balance -= dy;
                        }
                        balance += dx;
                        y += incy;
                }
                LCD_plot(x, y,lcolor);
        }
}


void LCD_rectangle(int x1,int y1,int x2,int y2, uint32_t c)
{
	LCD_line(x1,y1,x2,y1,c); 
	LCD_line(x2,y1,x2,y2,c); 
	LCD_line(x2,y2,x1,y2,c); 
	LCD_line(x1,y2,x1,y1,c); 
}


/* simple 3D function plotting */

void paraboloide(int PERSP,float SCALE,int TRAZOSX,int TRAZOSY,float X0,float X1,float Y0,float Y1,uint32_t pcolor)
{
auto int xx,yy;
auto float x,y,z;
auto float persp,xt,zt;
auto float h1,h2,h3,h4,cs,si;

	persp=PERSP*0.01745329;			// convert to radians
	cs=cos(persp);
	si=sin(persp);
	h1=(X1-X0)/159;
	h2=(Y1-Y0)/(float)(TRAZOSX-1);
	h3=(X1-X0)/(float)(TRAZOSY-1);
	h4=(Y1-Y0)/159;

	for(y=Y0;y<=Y1;y+=h2)
		for(x=X0;x<=X1;x+=h1){
			z=x*x/4+y*y/9-15;
			xt=x-y*cs;
			zt=z-y*si;
			xx=320+(int)(SCALE*xt);
			yy=240-(int)(SCALE*zt);
			LCD_plot(xx,yy,pcolor);
		}
	for(x=X0;x<=X1;x+=h3)
		for(y=Y0;y<=Y1;y+=h4) {
			z=x*x/4+y*y/9-15;
			xt=x-y*cs;
			zt=z-y*si;
			xx=320+(int)(SCALE*xt);
			yy=240-(int)(SCALE*zt);
			LCD_plot(xx,yy,pcolor);
		}
}

void elipsoide(int PERSP,float SCALE,int TRAZOSX,int TRAZOSY,float X0,float X1,float Y0,float Y1,uint32_t pcolor)
{
auto int xx,yy;
auto float x,y;
auto float persp,xt,yt,zt;
auto float h1,h2,h3,h4,cs,si;

	persp=PERSP*0.01745329;			// convert to radians
	cs=cos(persp);
	si=sin(persp);
	h1=(X1-X0)/159;
	h2=(Y1-Y0)/(float)(TRAZOSX-1);
	h3=(X1-X0)/(float)(TRAZOSY-1);
	h4=(Y1-Y0)/159;

	for(y=Y0;y<=Y1;y+=h2)
		for(x=X0;x<=X1;x+=h1){
			xt=cos(x)*cos(y);
			yt=sin(x)*cos(y);
			zt=sin(y);
			xt-=yt*cs;
			zt-=yt*si;
			xx=320+(int)(SCALE*xt);
			yy=240-(int)(SCALE*zt);
			LCD_plot(xx,yy,pcolor);
		}
	for(x=X0;x<=X1;x+=h3)
		for(y=Y0;y<=Y1;y+=h4) {
			xt=cos(x)*cos(y);
			yt=sin(x)*cos(y);
			zt=sin(y);
			xt-=yt*cs;
			zt-=yt*si;
			xx=320+(int)(SCALE*xt);
			yy=240-(int)(SCALE*zt);
			LCD_plot(xx,yy,pcolor);
		}
}

#include "6X8L.lib"
#include "12X16L.lib"

typedef struct {
	uint8_t *font;
	uint8_t	lpc;           // lines per character
	uint8_t	Bpcl;          // bytes per character line (1 or 2)
	uint8_t	bpcl;          // bits per character line
} FontInfo;

const static FontInfo fontinfo[]={
{Font6x8,8,1,6},
{Font12x16,16,2,12}
};

void LCD_putchr(uint16_t font,uint16_t row,uint16_t col,char chr ,uint32_t fcolor,long bcolor)
{
int i,j,ii,jj;
uint8_t *address;
uint16_t data;

	i=fontinfo[font].lpc;			// lines per character
	ii=fontinfo[font].Bpcl;			// bytes per character line
	jj=fontinfo[font].bpcl;			// bits per character line
	address=&(fontinfo[font].font[i*ii*(chr-0x20)]);
	LCD_setwindow (col,col+jj-1,row,row+i-1);
	LCD_WriteCmd(0x2C);
	while(i--){
   		data=*address;			// read first byte
		data<<=8;			// move to high part
		data+=address[1];		// read (possible) second byte in low part
		address+=ii;
		j=jj;
		while(j--){
			if(data&0x8000)
				LCD_WritePixel(fcolor);
			else {
				if(bcolor>=0){				// bcolor =-1 => transparent
					LCD_WritePixel((uint32_t)bcolor);
				}
				else {
					LCD_WritePixel(BLACK);
				}
      		}
        	data<<=1;
      	}
	}
}

void LCD_printat(uint16_t font,uint16_t row,uint16_t col,char *ptr,uint32_t fcolor,long bcolor)
{
	do {
		LCD_putchr (font,row,col,*ptr++,fcolor,bcolor);
		col+=fontinfo[font].bpcl;
	} 	while (*ptr);
}

void pattern(void)
{
uint16_t x,y;
uint8_t r,g,b;

	LCD_WriteCmd(0x2C);
  	for(y=0;y<480;y++,g++){
	   	if(y>255)
      			b++;
	for(x=0;x<640;x++,r++){
	   	if(x>255)
   		   	b++;
	      	LCD_WritePixel(MAKECOLOR(r,g,b));
	}
   }
}

//#include "tal_img.h"

/* MAIN PROGRAM */

main()
{
unsigned int i,y,oldy;
char string[5];


	LCD_fill(640,480,RED);		// R
	/* Setup I/O */
	TSB_PJ->CR   = 0xFF;                      // PJ defined as Outputs
	TSB_PI->CR   = 0xF8;                      // PI.3,4,5,6,7 defined as Outputs
	TSB_PI->DATA |= (1<<LCD_CSb)+(1<<LCD_WRb)+(1<<LCD_RDb)+(1<<LCD_RSTb);

	SysTick_Setup();                      // SysTick Setup

	LCD_init();

	while(1){
		LCD_fill(640,480,RED);		// R
		LCD_fill(640,480,GREEN);		// G
		LCD_fill(640,480,BLUE);		// B
		QUADS();
		pattern();
 		LCD_clear();
//		LCD_icon(160,80,320,305,(uint8_t *)tal);			// show gfx
		LCD_printat(1,420,320,"Tal Wilkenfeld",0x2045C2,0xB33018);
		MsDelay ( 15000 );						// wait 15 secs
 		LCD_clear();
		LCD_rectangle(0,0,639,479,GRAY);
 		for(i=0;i<16;i++) {
			LCD_rectangle(21*i+3,11*i+3,17*i+240,11*i+81,RED);
			LCD_rectangle(13*i+5,13*i+5,19*i+240,13*i+83,GREEN);
		}
		MsDelay ( 1000 );						// wait 1 secs
		LCD_clear();
		LCD_rectangle(10,5,128,45,RED);
		LCD_rectangle(8,3,130,47,GREEN);
		LCD_printat(0,20,20,"Cika Electronica",BLUE,0x000000);
		MsDelay ( 3000 );						// wait 5 secs
		LCD_clear();
		paraboloide(30,12,20,20,-8.0,8.0,-10.0,10.0,BLUE);
		LCD_printat(1,20,20,"Cika Electronica",GREEN,0xff0000);
		MsDelay ( 5000 );	 					// wait 5 secs
		elipsoide(45,160.0,30,30,0,6.28,-3.14,3.14,RED);
		LCD_printat(1,50,100,"Cika Electronica",GREEN,-1);
		MsDelay ( 5000 );						// wait 5 secs
		LCD_clear();
		LCD_line(0,240,639,240,GRAY);
		for(i=0;i<640;i++){
			oldy=y;
			y=(uint16_t)(240-239*sin((2*m_PI*i)/640)*cos((17*m_PI*i)/640));
			if(i==0)
				LCD_plot(i,y,GREEN);
			else
				LCD_line(i-1,oldy,i,y,GREEN);
		}
		for(i=0;i<640;i++){
			oldy=y;
			y=(uint16_t)(240+239*cos((5*m_PI*i)/640)*cos((m_PI*i)/640));
			if(i==0)
			  	LCD_plot(i,y,RED);
			else
				LCD_line(i-1,oldy,i,y,RED);
		}
		for(i=0;i<640;i++){
			oldy=y;
			y=(uint16_t)(240-239*sin(cos(tan(cos((2*m_PI*i)/640)))));
			if(i==0)
				LCD_plot(i,y,BLUE);
			else
				LCD_line(i-1,oldy,i,y,BLUE);
		}
		for(i=0;i<10;i++){
			LCD_line(64*i,235,64*i,245,GRAY);
			sprintf(string,"%d",i);
			LCD_printat(0,250,64*i,string,GRAY,-1);
		}
		LCD_printat(1,0,0,"Dermopirifilacion",WHITE,-1);
		LCD_printat(1,0,500,"Sensor A",RED,-1);
		LCD_printat(1,20,500,"Sensor B",GREEN,-1);
		LCD_printat(1,40,500,"Sensor C",BLUE,-1);
		MsDelay ( 10000 );						// wait 10 secs
		LCD_setwindow ( 0, 639, 0, 479);
	}
}
